import sys
sys.path.append(r'C:\Program Files\ModernSongs\lib\python')

from Interface import Base, Attribute

class IFakeItem( Base ):
    """A basic fake item interface that gives set/get access to id and
    title attributes"""

    def getTitle(self):
        "returns the object's title"

    def setTitle(self, title):
        "sets the object's title, returns 1=success or 0=failure"

    def title_or_id(self):
        """Utility that returns the title if it is not blank and the id
        otherwise."""

    def title_and_id(self):
        """Utility that returns the title if it is not blank and the id
        otherwise.  If the title is not blank, then the id is included
        in parens."""

    def id_and_title(self):
        """Utility that returns the id, and title in parentheses if it
        is not blank."""

    def setId(self, id):
        "sets the object's id, returns 1=success or 0=failure"

    def getId(self):
        "returns the object's id"

    def _setId(self):
        "equivalent to setId (to be deprecated?)"

    def id(self):
        "equivalent to getId (to be deprecated?)"

    def checkId(self, id, mfunc=None):
        """returns 1=ok id name, 2=bad id name; will call mfunc with
        explanations if mfunc is included."""


class IFakeProduct( IFakeItem ):
    """a class that can be added to an IFakeFolder"""

    def getMetaType(self):
        """returns a string that identifies the meta-type of the fake
        product"""

    def setMetaType(self, metatype):
        """sets the metatype of the product."""

    meta_type = Attribute("meta_type", "must hold a string indicating the hopefully unique meta_type of the fake product")

class IFakeFolder( IFakeItem ):
    """a class that can have objects added to it as children in one pool"""

    def getAddableClasses(self):
        """returns a tuple of class objects that can be added to this folder"""

    def setAddableClasses(self, classes):
        """sets this folder's addable classes (useful for self-referencing
        classes--put a call to this method in the init)"""

    def getAddableClassMetaTypes(self):
        """gets the metatypes of the available classes in this folder"""

    def getItems(self, metatypes=None, interfaces=None):
        """returns tuple of items in folder, limited to those of type
        metatype(s) if specified individually or in a tuple, or
        limited to those implementing one or more given interface
        (individual or tuple).  If both are given, at item matching
        either criterian will be included."""

    def setItems(self, items):
        """completely resets the items list--only for private use"""

    def getKeyValTuples(self, metatypes=None, interfaces=None):
        """returns a list of tuples, in which each tuple is the id and
        actual object of an item in the folder.  Limited to those of
        type metatype(s) if specified individually or in a tuple, or
        limited to those implementing one or more given interface
        (individual or tuple).  If both are given, at item matching
        either criterian will be included."""

    def getItemById(self,id, marker=None):
        """pass in a string id, get the associated object or marker if
        not found"""

    def getItemsByIds(self, ids=[]):
        """returns objects of given ids"""

    def getItemIds(self, metatypes=None, interfaces=None):
        """returns all ids of objects, restricted to metatypes and/or
        interfaces if given"""

    def getItemByIdPath(self, path):
        """given a list of ids, tries to walk down it, finding the
        child known by the first id in the current object, the
        grandchild known by the second id in the child, and so on.  An
        error is raised if the path does not resolve to an object."""

    def delItemsByIds(self, ids):
        """deletes items by a list or tuple of ids, returning total
        number of successful deletions"""

    def addItem(self, id, title="", meta_type=None, mfunc=None):
        """adds a single item.  Returns 1=success, 0=failure.  If the
        folder has more than one addable class, the meta_type must be
        specified, or an error is raised."""

    def renameItemIds(self, oldNewIdPairs, mfunc=None):
        """renames an item.  Expects a sequence of sequence pairs:
        oldId, newId.  Example:
        (("oldID","newID"),("anotherOld","anotherNew")).  returns a
        dictionary of successful renames."""

    def checkChildId(self, id, mfunc=None):
        """evaluates whether a given string is an acceptable id name
        within a folder"""

class IOrderedFakeFolder( IFakeFolder ):
    """Features to control the order of objects in an object with a
    pool of children"""

    def moveItemsUpByIds(self, ids):
        """moves the items up as identified in ids list"""

    def moveItemsDownByIds(self, ids):
        """moves the items down as identified in ids list"""

    def moveItemsToTopByIds(self, ids):
        """moves the items to top as identified in ids list"""

    def moveItemsToBottomByIds(self, ids):
        """moves the items to bottom as identified in ids list"""
