"""a fake folder for fake products"""

from OFS.SimpleItem import SimpleItem # Item
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, DTMLFile
from Acquisition import aq_base
import Fakers, ZFakeItem, ZFakeProduct, IZFakers, utils

class ZFakeFolder(Fakers.FakeFolder, ZFakeItem.ZFakeItem): #, Navigation): #, Tabs):
    """a fake folder for fake products"""

    def __init__(self, id, title=""):
        self._items=()
        ZFakeItem.ZFakeItem.__init__(self, id, title)

    __implements__=IZFakers.IZFakeFolder

    meta_type="Fake Folder"

    isPrincipiaFolderish=1

    security = ClassSecurityInfo()
    security.setDefaultAccess("allow")

    addable_classes=(ZFakeProduct.ZFakeProduct,) # is the magic hook used to list, yes, the classes
    # you can add to this fake folder.  A tuple of classes.

    ## security declarations for non-Zope base class

    security.declareProtected( "Access contents information", 'getItems' )
    security.declarePrivate( 'setItems' )
    security.declareProtected( "Access contents information", 'getKeyValTuples' )
    security.declareProtected( "Access contents information", 'getAddableClasses' )
    security.declarePrivate( 'setAddableClasses' )
    security.declareProtected( "Access contents information", 'getItemById' )
    security.declareProtected( "Delete objects", 'delItemsByIds' )
    security.declareProtected( "Access contents information", 'getItemsByIds' )
    security.declareProtected( "Add Fake Products", 'addItem' )
    security.declareProtected( 'Manage properties', 'renameItemIds' ) # 'View management screens' for below

    ## Acquisition-based functions

    security.declareProtected( "Access contents information", 'getAcquiredItems' )
    def getAcquiredItems(self, metatypes=None, interfaces=None):
        """returns list of items with acquisition set"""
        if not metatypes and not interfaces:
            return self._items
        if type(metatypes) != type(()):
            metatypes=(metatypes,)
        if type(interfaces) != type(()):
            interfaces=(interfaces,)
        ret=[]
        for x in self._items:
            if metatypes and x.getMetaType() in metatypes:
                ret.append(x.__of__(self))
            elif interfaces:
                for ntrfc in interfaces:
                    if ntrfc.isImplementedBy(x):
                        ret.append(x.__of__(self))
                        break
        return ret

    security.declareProtected( "Access contents information", 'getAcquiredKeyValTuples' )
    def getAcquiredKeyValTuples(self, metatypes=None, interfaces=None):
        """returns a list of tuples, in which each tuple is one id and
        associated actual *acquired* object from the folder"""
        return [(x.getId(), x) for x in self.getAcquiredItems(metatypes, interfaces)]

    security.declareProtected( "Access contents information", 'getAcquiredItemById' )
    def getAcquiredItemById(self, id, marker=None):
        """pass in a string id, get the associated object or None if not
        found"""
        item=self.getItemById(id)
        if item: return item.__of__(self)
        return marker

    security.declareProtected( "Access contents information", 'getAcquiredItemByIdPath' )
    def getAcquiredItemByIdPath(self, path):
        """given a list of ids, tries to walk down it, finding the
        child known by the first id in the current object, the
        grandchild known by the second id in the child, and so on.  An
        error is raised if the path does not resolve to an object.
        Final object is acquired by all parents."""
        obj=self
        for id in path:
            if obj is utils.marker or not IZFakers.IZFakeFolder.isImplementedBy(obj):
                raise LookupError, "Path %s not found in ZFakeFolder %s" % ("/".join(path), self.getId())
            obj=obj.getAcquiredItemById(id, utils.marker)
        if obj is utils.marker:
            raise LookupError, "Path %s not found in ZFakeFolder %s" % ("/".join(path), self.getId())
        return obj

    security.declareProtected( "Access contents information", 'getAcquiredItemsByIds' )
    def getAcquiredItemsByIds(self, ids=[]):
        """returns objects of given ids"""
        return [x.__of__(self) for x in self.getItems() if x.getId() in ids]

    security.declarePrivate( 'checkChildId' )
    def checkChildId(self, id, mfunc=None):
        """evaluates whether a given string is an acceptable id name within a folder"""
        if hasattr(aq_base(self), id):
            if mfunc: mfunc("'%s' is a reserved id" % id)
            return 0
        elif self.getItemById(id):
            if mfunc: mfunc("an item identified as '%s' is already in this folder" % id)
            return 0
        elif " " in id:
            if mfunc: mfunc("this folder disallows ids with spaces in them such as '%s'" % id)
            return 0
        else:
            return 1

    ## Management

    specific_manage_options=(
        {'label':'Contents', 'action':'manage_contents', 'help':('Fakers','FakeFolder_Edit.stx')},
        )
    generic_manage_options=ZFakeItem.ZFakeItem.manage_options
    manage_options=(
        specific_manage_options + generic_manage_options
        )

    security.declareProtected( 'View management screens', 'manage_contents' )
    # manage_main =
    manage_contents = DTMLFile('dtml/manage_contents', globals())

    security.declareProtected( 'View management screens', 'manage_addItemForm' )
    manage_addItemForm = DTMLFile('dtml/add_item', globals())

    security.declareProtected('View management screens', 'manage_renameItemsForm' )
    manage_renameItemsForm=DTMLFile('dtml/rename_items', globals())

    security.declareProtected( 'View management screens', 'manage_addItem' )
    def manage_addItem(self, id, title="", meta_type="", REQUEST=None):
        """adds an item to the folder; returns HTML or text confirmation"""
        msg=[]
        res=self.addItem(id, title, meta_type, mfunc=msg.append)
        if not res:
            if REQUEST is not None:
                return self.manage_contents(self, REQUEST, message=msg or 'Sorry: the id you chose could not be used')
            else: return msg or 'Sorry: the id you chose could not be used'
        if REQUEST is not None:
            return self.manage_contents(self, REQUEST, message="Item %s added at bottom of list." % id)
        return "Item %s added at bottom of list." % id

    security.declareProtected( 'View management screens', 'manage_renameItems' )
    def manage_renameItems(self, ids=[], new_ids=[], REQUEST=None):
        """Rename several items"""
        if len(ids) != len(new_ids):
            raise 'Bad Request','Please rename each listed virtual item; deleting must be done separately.'
        msg=[]
        result=self.renameItemIds([(ids[x], new_ids[x]) for x in range(len(ids))], mfunc=msg.append)
        if REQUEST is not None:
            return self.manage_contents(self, REQUEST, update_menu=1, message="%d items renamed%s" % (len(result), (msg and (": "+", ".join(msg)) or "")))
        return "%d items renamed%s" % (len(result), (msg and (": "+", ".join(msg)) or ""))

    security.declareProtected( 'View management screens', 'manage_delItems' )
    def manage_delItems(self, ids=[], REQUEST=None):
        """deletes selected items; returns HTML or text confirmation"""
        if type(ids) is type(''): ids=[ids]
        if not ids:
            if REQUEST is not None:
                return self.manage_contents(self, REQUEST, message='No items were specified!')
            else: return 'No items were specified!'
        attempt=len(ids)
        final=self.delItemsByIds(ids)
        if REQUEST is not None:
            return self.manage_contents(self, REQUEST, update_menu=1, message="Performed %d of %d requested deletions." % (final, attempt)) # update_menu? The pertinent code isn't in my dtml page but I'll leave it in case I want to add it in for some reason.
        return "Performed %d of %d requested deletions." % (final, attempt)

    def __bobo_traverse__(self, request, name): # there is another function I should be using for this normal stuff...Probably the mapping functions...
        '''Traversing away'''
        x= self.getAcquiredItemById(name)
        if x: return x # .__of__(self)
        if hasattr(self, name): return getattr(self, name)
        return None

    def tpValues(self):
        # Return a list of subobjects, used by tree tag.
        r=[]
        obj_ids=self.getItemIds()
        obj_ids.sort()
        for id in obj_ids:
            o=self.getAcquiredItemById(id)
            if hasattr(o, 'isPrincipiaFolderish') and \
               o.isPrincipiaFolderish:
                r.append(o)
        return r


manage_addZFakeFolderForm=DTMLFile('dtml/addZFakeFolder', globals())

def manage_addZFakeFolder(self, id, REQUEST=None, title="", **ignored):
    """Constructor for ZFakeFolder"""
    obj = ZFakeFolder(id, title)
    #obj.setId(str(id))
    #if title: obj.setTitle(title)
    self._setObject(id, obj)
    if REQUEST:
        return self.manage_main(self,REQUEST, update_menu=1)
    else:
        return 1

constructors = (
  ('manage_addZFakeFolderForm', DTMLFile('www/addZFakeFolder', globals())),
  ('manage_addZFakeFolder', manage_addZFakeFolder),
)


InitializeClass(ZFakeFolder)
