"""The root mix-in for anything that is to be added to a fake folder"""

from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
import Fakers, ZFakeItem, IZFakers

class ZFakeProduct(Fakers.FakeProduct, ZFakeItem.ZFakeItem):
    """The root mix-in for anything that is to be added to a fake folder"""

    def __init__(self, id, title):
        ZFakeItem.ZFakeItem.__init__(self, id, title)

    security = ClassSecurityInfo()

    __implements__=IZFakers.IZFakeProduct

    manage_options=ZFakeItem.ZFakeItem.manage_options

    security.declarePublic("getMetaType")
    security.declareProtected("Manage properties", "setMetaType")

    security.declarePublic("getIconURL")
    def getIconURL(self):
        """returns the Zope path to an icon that can represent this
        fake product."""
        return "/"+self.icon

    security.declareProtected( 'Manage properties', 'setIconURL' )
    def setIconURL(self, iconURL, mfunc=None):
        """sets the icon URL"""
        try:
            img=self.unrestrictedTraverse(iconURL)
            if img.meta_type in ("Image", "Filesystem Image", "Help Image", "Portal Image"):
                self.icon=iconURL[1:]
                return 1
            if mfunc:
                mfunc("%s is not an image" % iconURL)
            return 0
        except:
            if mfunc: mfunc("%s not found" % iconURL)
            return 0


    icon="misc_/Fakers/FakeProduct"
    # meta_type is in FakeProduct--also necessary to set!

InitializeClass(ZFakeProduct)
