
marker=[]

def moveItemsUp(items, ids):
    """a utility function used by the ordering functions. Don't
    call this ever: this is private.  Use the actual ordering
    functions instead. Just so you know, you pass in a tuple or
    list of objects in 'items' and a tuple or list of string ids
    in 'ids'.  It returns a list of items (yes, a list) in which
    the items with ids matching those in the ids list have been
    moved up by one, if possible.  This is used by the MoveUp and
    MoveDown ordering functions."""

    res=[]
    last=None
    for item in items:
        if item.getId() in ids:
            res.append(item)
        else:
            if last: res.append(last)
            last=item
    if last: res.append(last)
    return res

def separateItems(items, ids):
    """a utility function: private, hands off.  Pass in a list or
    tuple of objects in 'items' and a list or tuple of string ids
    in 'ids', get back a tuple of two lists of objects: the first
    list contains those items that have ids matching one in the
    ids list, the second holds the remainder.  The function
    retains the ordering of items within the two groups. It is
    used by the ToTop and ToBottom functions."""

    mem=[]
    out=[]
    for item in items:
        if item.getId() in ids: mem.append(item)
        else: out.append(item)
    return mem, out
